%%%%%%%%%%
% STAR.M %
%%%%%%%%%%

% This program is to calculate the approximate LM test for a STAR model 

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/


function [s3,ps3]=star(dat,p,omit)

%Construct Data%
t=length(dat)-p;

if omit==0
    k=p;
else
    ro=length(omit);
    k=p-ro;
end;

k1=k+1;
y=dat(1+p:t+p);
x=ones(t,1);

j=1;
while j<=p
    sumc=0;
    for i=1:length(omit)
        if omit(i)==j
            sumc=sumc+1;
        end;
    end;
    if sumc==0
        x=[x,dat(1+p-j:t+p-j)];
    end;
    j=j+1;
end;

%Null Model%
u=y-x*(inv(x'*x)*(x'*y));
uu=u'*u;
f=[x,(x(:,2:k1).*x(:,2:k1)),x(:,2:k1).*x(:,2:k1).*x(:,2:k1)];

if k>1
    j=2;
    while j<=k1
        j1=j+1;
        while j1<=k1;
            f=[f,(x(:,j).*x(:,j1))];
            j1=j1+1;
        end;
        j=j+1;
    end;
end;

e=u-f*(inv(f'*f)*(f'*u));
ee=e'*e;
s3=t*(uu-ee)/uu;
ps3=1-chi2cdf(s3,(k1*k/2)+k);
fprintf('Star Statistics:   %f         %f',s3,ps3);

%LATEX%
ttex=fopen('out.tex','a+');
fprintf(ttex,'\\begin{tabular}{l l l }\n');
fprintf(ttex,'\\textbf{Star Statistics:}&%f&%f\\\\\n',s3,ps3);
fprintf(ttex,'\\end{tabular}\n');
fprintf(ttex,'\\vspace{.2cm}\\\\\n');
fclose(ttex);


